#include "i2c.h"
#define DS1307_ID 0xD0	
#define SEC   0x00
#define	MIN   0x01	
#define HOUR  0x02
#define DAY	  0x03
#define DATE  0x04
#define MONTH 0x05
#define YEAR  0x06

sbit UP=P3^5;
sbit DOWN=P3^6;
sbit SET=P3^4;
sbit RATE=P3^7;

code const unsigned char * MonthStr[12]={{"Jan"},
										{"Feb"},
										{"Mar"},
										{"Apr"},
										{"May"},
										{"Jun"},
										{"Jul"},
										{"Aug"},
										{"Sep"},
										{"Oct"},
										{"Nov"},
										{"Dec"}};
code const unsigned char * DayStr[7]  ={{"Sun"},
				   						{"Mon"},
				   						{"Tue"},
				   						{"Wen"},
				   						{"Thu"},
				   						{"Fri"},
				   						{"Sat"}};

unsigned char DS1307_get(unsigned char addr)
{
	unsigned char ret;	
	I2C_start();            /* Start i2c bus */
	I2C_write(DS1307_ID);   /* Connect to DS1307 */
	I2C_write(addr);		/* Request RAM address on DS1307 */
	I2C_start();			/* Start i2c bus */
	I2C_write(DS1307_ID+1);	/* Connect to DS1307 for Read */
	ret = I2C_read();		/* Receive data */
	I2C_stop();				/* Stop i2c bus */
	return ret;
 
}

void DS1307_settime(unsigned char dy,unsigned char hh, unsigned char mm, unsigned char ss)
{
	I2C_start(); 

	I2C_write(DS1307_ID);	/* connect to DS1307 */ 
	I2C_write(0x00);		/* Request RAM address at 00H */ 
   	I2C_write(ss);			/* Write sec on RAM address 00H */
	I2C_write(mm);			/* Write min on RAM address 01H */
	I2C_write(hh);
	I2C_write(dy);				/* Write hour on RAM address 02H */
   	I2C_stop();           	/* Stop i2c bus */
}

void DS1307_setdate(unsigned char dd, unsigned char mm, unsigned char yy)
{
 	I2C_start();	
  	I2C_write(DS1307_ID);	/* connect to DS1307 */
	I2C_write(0x04);		/* Request RAM address at 04H */ 	
	I2C_write(dd);			/* Write date on RAM address 04H */
	I2C_write(mm);			/* Write month on RAM address 05H */
	I2C_write(yy);			/* Write year on RAM address 06H */
	I2C_stop();				/* Stop i2c bus */
}

void send2lcd(unsigned char value)
{
	unsigned char buf = 0;	
	
	buf = value & 0xF0;		/* Filter for high byte */
	buf = (buf>>4)|(0x30);		/* Convert  to ascii code */

	WriteCharLCD(buf);			/* Show on LCD */

	buf = value & 0x0F;		/* Filter for low byte */
	buf = buf | 0x30;       	/* Convert to ascii code */

	WriteCharLCD(buf);			/* Show on LCD */
}
void send2lcd1(unsigned char value)
{
	unsigned char buf = 0;	
	
	buf = value & 0xF0;		/* Filter for high byte */
	buf = (buf>>4)|(0x30);		/* Convert  to ascii code */

	WriteCharLCD1(buf);			/* Show on LCD */

	buf = value & 0x0F;		/* Filter for low byte */
	buf = buf | 0x30;       	/* Convert to ascii code */

	WriteCharLCD1(buf);			/* Show on LCD */
}
char * Int2Month(unsigned char month)
{
  return MonthStr[(month)-1];
}
unsigned char BCD2HEX(unsigned char bcd)
{
	
	return( ((bcd >> 4)*10) + (bcd & 0x0f) );
}
unsigned char INT2BCD(unsigned char x)
{
	unsigned char temp=0;
	temp=x%10;
	x/=10;
	temp=(x<<4)|temp;
	return temp;		
	
}
char * Int2Day(unsigned char day)
{
	return DayStr[day-1];
}
void dispDateTime(void)
{
 	unsigned char ss,mm,hh,dy,dd,mh,yy;
	ss = DS1307_get(SEC);
	mm = DS1307_get(MIN);
	hh = DS1307_get(HOUR);
	dy = DS1307_get(DAY);
	dd = DS1307_get(DATE);
	mh = DS1307_get(MONTH);
	yy = DS1307_get(YEAR);
	DisplayStrLCD(0,"Date:");
	send2lcd(dd);//display date 
	WriteCharLCD('-');
	DisplayStrLCD(0,Int2Month(BCD2HEX(mh)));	/* Show month on LCD */
	WriteCharLCD('-');
	DisplayStrLCD(0,"20");
	send2lcd(yy);		/* Show year on LCD */
	GoToLCD(0,2,1);
	DisplayStrLCD(0,"Time:");
	send2lcd(hh);
	WriteCharLCD(':');
	send2lcd(mm);
}
void SetTime(void)
{
 	unsigned char min,hr,day,dat,mon,yr,set;
	ClearScreenLCD(1);
	DelayMS(50);
	GoToLCD(1,1,1);
	DisplayStrLCD(1,"Enter Minutes:");
	GoToLCD(1,2,1);
	set=BCD2HEX(DS1307_get(MIN));//set minutes
	while(SET==1)
	{
		GoToLCD(1,2,9);
		if(set<10)
		{
			WriteCharLCD1('0');	
		}
		if(set>59 || set<0)
		{
			set=0;
		}
		DisplayIntegerLCD(1,set);
		if(UP==0)
		{	if(set==59)
				set=0;
			else
				set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=59;
			else
				set--;
			DelayMS(200);
		}
		
		if(SET==0)
		{
			break;
		}
	}
	min=INT2BCD(set);
	//set hours
	GoToLCD(1,1,1);
	SET=UP=DOWN=1;
	ClearScreenLCD(1);
	DelayMS(50);
	DisplayStrLCD(1,"Enter Hours:");
	set=BCD2HEX(DS1307_get(HOUR));
	
	{
		GoToLCD(1,2,1);
		while(SET==1)
		{
			GoToLCD(1,2,1);
			if(set<10)
			{
				WriteCharLCD1('0');	
				
			}
			if(set>23 || set<0)
			{
				set=0;
			}
			DisplayIntegerLCD(1,set);
			if(UP==0)
			{	
				if(set==23)
					set=0;	
				else
					set++;
				DelayMS(200);
			}
			if(DOWN==0)
			{
				if(set==0)
					set=23;
				else
					set--;
				DelayMS(200);
			}
		}
		hr=INT2BCD(set);
	}
	
	ClearScreenLCD(1);
	DelayMS(200);
	GoToLCD(1,1,1);
	DisplayStrLCD(1,"Enter Day:");
	set=BCD2HEX(DS1307_get(DAY));
	while(SET==1)
	{
		GoToLCD(1,2,1);
		if(set>7 || set<1)
		{
			set=1;
		}
		DisplayStrLCD(1,Int2Day(set));
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==1)
				set=7;
			else
				set--;
			
			DelayMS(200);
		}
	}
	day = INT2BCD(set);
	ClearScreenLCD(1);
	//set date
	DelayMS(200);

	GoToLCD(1,1,1);
	DisplayStrLCD(1,"Enter Date:");
	set=BCD2HEX(DS1307_get(DATE));
	while(SET==1)
	{
		GoToLCD(1,2,1);
		if(set<10)
		{
			WriteCharLCD1('0');	
			
		}
		if(set>31 || set<0)
		{
			set=1;
		}
		DisplayIntegerLCD(1,set);
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=31;
			else
				set--;
			DelayMS(200);
		}
		
		
	}
	dat= INT2BCD(set);
	ClearScreenLCD(1);
	DelayMS(200);
	GoToLCD(1,1,1);
	DisplayStrLCD(1,"Enter Month:");
	set=BCD2HEX(DS1307_get(MONTH));
	while(SET==1)
	{
		GoToLCD(1,2,1);
		if(set>12 || set<=0)
		{
			set=1;
		}
		if(set==0)
				set=12;
		DisplayStrLCD(1,Int2Month(set));
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==1)
				set=12;
			else
				set--;
			DelayMS(200);
		}
		
		
	}
	mon = INT2BCD(set);
	ClearScreenLCD(1);
	DelayMS(200);
	GoToLCD(1,1,1);
	DisplayStrLCD(1,"Enter year:");
	GoToLCD(1,2,1);
	DisplayStrLCD(1,"20");
	set=BCD2HEX(DS1307_get(YEAR));
	while(SET==1)
	{
		GoToLCD(1,2,3);
		if(set>99 || set<10)
		{
			set=10;
		}
		DisplayIntegerLCD(1,set);
		if(UP==0)
		{
			set++;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			
			if(set==0)
				set=99;
			else
				set--;
			DelayMS(200);
		}
	}
	yr=INT2BCD(set);
	ClearScreenLCD(1);
	DelayMS(50);
   	DS1307_settime(day,hr,min,0x00);
	DS1307_setdate(dat,mon,yr);
	DisplayStrLCD(1,"Saved Successfully");
	
}
void PutTime(unsigned char slot)
{
 	unsigned char dat[4];
	unsigned char mm,hh;
   	mm = DS1307_get(MIN);
	hh = DS1307_get(HOUR);
	
	GoToLCD(0,2,1);
	DisplayStrLCD(0,"ChkIn:");
	send2lcd(hh);
	WriteCharLCD(':');
	send2lcd(mm);

	dat[0] = BCD2HEX(mm);
	dat[1] = BCD2HEX(hh);
	dat[2] = 0;
	dat[3] = 0;
	
	if(slot==1)
	{
		   WrToROM(dat,0x00); //@ 0x00 location the in time for slot 1 is stored
		   DelayMS(50);
		   WrToROM("A",0x40); //A stands for alloted here 
	}
	if(slot==2)
	{
		   WrToROM(dat,0x10);//@ 0x10 location the in time for slot 1 is stored
		   DelayMS(50);
		   WrToROM("A",0x41);//A stands for alloted here
	}
	if(slot==3)
	{
		   WrToROM(dat,0x20);//@ 0x20 location the in time for slot 1 is stored
		   DelayMS(50);
		   WrToROM("A",0x42);//A stands for alloted here
	}
	if(slot==4)
	{
		   WrToROM(dat,0x30);//@ 0x30 location the in time for slot 1 is stored
		   DelayMS(50);
		   WrToROM("A",0x43);//A stands for alloted here
	}

}
void SetRate(void)
{
 	unsigned char set=20;
	unsigned char dat[2]={0,0};
	
	RdFromROM(dat,0x50,1);
	if(dat[0]>0)
	{
		set=dat[0];
	}
	ClearScreenLCD(1);
	DisplayStrLCD(1,"Set Parking Charges");
	while(RATE==1)
	{
		GoToLCD(1,2,1);
		DisplayIntegerLCD(1,set);
		DisplayStrLCD(1,"    ");
		if(UP==0)
		{
			if(set>250)
			{
				set=250;
			}
			set+=5;
			DelayMS(200);
		}
		if(DOWN==0)
		{
			if(set<0)
			{	
				set=0;			
			}
			set-=5;;
			DelayMS(200);
		}
	}
	dat[0] = set;
	WrToROM(dat,0x50);
	ClearScreenLCD(1);
	DisplayStrLCD(1,"Saved");
	DelayMS(1000);
}
int getRate(void)
{
   unsigned char dat;
   RdFromROM(&dat,0x50,1);
   return (dat);
}